//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/BackgroundItems.h
//! @brief     Defines BackgroundItem classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_ITEM_BACKGROUNDITEMS_H
#define BORNAGAIN_GUI_MODEL_ITEM_BACKGROUNDITEMS_H

#include "GUI/Model/Types/DoubleProperty.h"
#include <memory>

class IBackground;
class Serializer;

class BackgroundItem {
public:
    virtual ~BackgroundItem() = default;

    virtual std::unique_ptr<IBackground> createBackground() const = 0;
    virtual void serialize(Serializer& s) {}
};

class NoBackgroundItem : public BackgroundItem {
public:
    std::unique_ptr<IBackground> createBackground() const override;
};

class ConstantBackgroundItem : public BackgroundItem {
public:
    ConstantBackgroundItem();
    std::unique_ptr<IBackground> createBackground() const override;
    void serialize(Serializer& s) override;

    DOUBLE_PROPERTY(backgroundValue, BackgroundValue);
};

class PoissonNoiseBackgroundItem : public BackgroundItem {
public:
    std::unique_ptr<IBackground> createBackground() const override;
};

#endif // BORNAGAIN_GUI_MODEL_ITEM_BACKGROUNDITEMS_H
