//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/FootprintItems.cpp
//! @brief     Implements FootprintItem classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Item/FootprintItems.h"
#include "Device/Beam/FootprintGauss.h"
#include "Device/Beam/FootprintSquare.h"

std::unique_ptr<IFootprintFactor> FootprintNoneItem::createFootprint() const
{
    return {};
}

FootprintGaussianItem::FootprintGaussianItem(double value)
{
    m_gaussianFootprintValue.init("Width ratio", "The ratio of beam and sample full widths", value,
                                  Unit::unitless, 3, RealLimits::nonnegative(), "ratio");
}

std::unique_ptr<IFootprintFactor> FootprintGaussianItem::createFootprint() const
{
    return std::make_unique<FootprintGauss>(m_gaussianFootprintValue);
}

FootprintSquareItem::FootprintSquareItem(double value)
{
    m_squareFootprintValue.init("Width ratio", "The ratio of beam and sample full widths", value,
                                Unit::unitless, 3, RealLimits::nonnegative(), "ratio");
}

std::unique_ptr<IFootprintFactor> FootprintSquareItem::createFootprint() const
{
    return std::make_unique<FootprintSquare>(m_squareFootprintValue);
}
