//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Item/PointwiseAxisItem.h
//! @brief     Defines pointwise axis item
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_ITEM_POINTWISEAXISITEM_H
#define BORNAGAIN_GUI_MODEL_ITEM_POINTWISEAXISITEM_H

#include "GUI/Model/Item/AxesItems.h"

class PointwiseAxis;
class SpecularInstrumentItem;

//! Item for non-uniform axis with specified coordinates.
class PointwiseAxisItem : public BasicAxisItem {
private:
    static constexpr auto P_NATIVE_AXIS_UNITS{"NativeAxisUnits"};

public:
    static constexpr auto M_TYPE{"PointwiseAxis"};

    explicit PointwiseAxisItem();
    ~PointwiseAxisItem() override;

    // setters, getters
    void init(const IAxis& axis, const QString& units_label);
    const IAxis* axis() const;
    QString getUnitsLabel() const;

    std::unique_ptr<IAxis> createAxis(double scale) const override;

    void updateIndicators();

    QByteArray serializeBinaryData() const override;
    void deserializeBinaryData(const QByteArray& data) override;

private:
    bool checkValidity() const;

    //! Sets internal m_instrument to containing SpecularInstrumentItem
    void findInstrument();

    const SpecularInstrumentItem* m_instrument;
    std::unique_ptr<IAxis> m_axis;
};

#endif // BORNAGAIN_GUI_MODEL_ITEM_POINTWISEAXISITEM_H
