//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Model/InstrumentModel.h
//! @brief     Defines class InstrumentModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MODEL_INSTRUMENTMODEL_H
#define BORNAGAIN_GUI_MODEL_MODEL_INSTRUMENTMODEL_H

#include "GUI/Model/Model/SessionModel.h"
#include <functional>

class InstrumentItem;
class Instrument2DItem;
class Serializer;

class InstrumentModel {
public:
    ~InstrumentModel();

    void clear();

    void serialize(Serializer& s);

    //! Inserts a deep copy (also of any non xml data in a pointwise axis)
    //! The id will not be copied, but a new unique one will be created
    //! Returns the newly created instrument.
    InstrumentItem* insertCopy(const InstrumentItem& source);

    QVector<InstrumentItem*> instrumentItems() const;
    QVector<Instrument2DItem*> instrument2DItems() const;

    InstrumentItem* findInstrumentById(const QString& instrumentId) const;
    bool instrumentExists(const QString& instrumentId) const;

    template <typename T> T* addInstrument();
    void removeInstrument(InstrumentItem* instrument);

    QString suggestInstrumentName(const QString& baseName) const;

    QVector<InstrumentItem*>
    instrumentItems(const std::function<bool(const InstrumentItem*)>& accept) const;

    QStringList instrumentNames() const;

private:
    QVector<InstrumentItem*> m_instruments;
};

template <typename T> T* InstrumentModel::addInstrument()
{
    auto* t = new T();
    m_instruments << t;
    return t;
}

#endif // BORNAGAIN_GUI_MODEL_MODEL_INSTRUMENTMODEL_H
