//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/InstrumentView.h
//! @brief     Defines class InstrumentView
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_INSTRUMENT_INSTRUMENTVIEW_H
#define BORNAGAIN_GUI_VIEW_INSTRUMENT_INSTRUMENTVIEW_H

#include <QWidget>

class InstrumentItem;
class InstrumentListView;
class ProjectDocument;
class QScrollArea;
class QLabel;
class QLineEdit;

class InstrumentView : public QWidget {
    Q_OBJECT

public:
    InstrumentView(QWidget* parent, ProjectDocument* document);

protected:
    void showEvent(QShowEvent*) override;
    void hideEvent(QHideEvent*) override;

private:
    void createWidgetsForCurrentInstrument();
    void onInstrumentNameEditingFinished();
    void onInstrumentChangedByEditor();
    void onInstrumentChangedFromExternal(const InstrumentItem* instrument);

private:
    InstrumentListView* m_instrumentListView;
    ProjectDocument* m_document;
    QLineEdit* m_nameLineEdit;
    QLabel* m_typeLabel;
    QScrollArea* m_scrollArea;
};

#endif // BORNAGAIN_GUI_VIEW_INSTRUMENT_INSTRUMENTVIEW_H
