//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Tool/WidgetUtils.h
//! @brief     Defines GUI::Util namespace
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2022
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_TOOL_WIDGETUTILS_H
#define BORNAGAIN_GUI_VIEW_TOOL_WIDGETUTILS_H

#include "GUI/Model/Group/SelectionDescriptor.h"
#include "GUI/Model/Types/Unit.h"
#include "GUI/View/PropertyEditor/CustomEventFilters.h"
#include <QComboBox>
#include <variant>

class QSpinBox;
class UIntDescriptor;
class DoubleSpinBox;
class DoubleDescriptor;
class QFormLayout;

namespace GUI::Util {

// #bamigration docu
//! Changes in the combobox will be notified to the SelectionDescriptor already. The addition slot
//! exists to be notified about an already executed change.
template <typename T>
QComboBox* createSelectionCombo(QWidget* parent, const SelectionDescriptor<T> d,
                                std::function<void(int)> slot = nullptr)
{
    auto* combo = new QComboBox(parent);
    combo->addItems(d.options);
    combo->setMaxCount(d.options.size());
    combo->setToolTip(d.tooltip);
    combo->setCurrentIndex(d.currentIndex());
    WheelEventEater::install(combo);

    QObject::connect(combo, qOverload<int>(&QComboBox::currentIndexChanged),
                     [=](int newIndex) { d.setCurrentIndex(newIndex); });

    if (slot)
        QObject::connect(combo, qOverload<int>(&QComboBox::currentIndexChanged),
                         [=](int newIndex) { slot(newIndex); });

    return combo;
}

// #bamigration docu
QSpinBox* createSpinBox(QWidget* parent, const UIntDescriptor& d);
QSpinBox* createSpinBox(QWidget* parent, const UIntDescriptor& d, std::function<void(uint)> slot);

QSpinBox* createSpinBox(QFormLayout* parentLayout, const UIntDescriptor& d);

DoubleSpinBox* createSpinBox(QFormLayout* parentLayout, const DoubleDescriptor& d);

//! No trailing ':'
QString labelWithUnit(const QString& label, std::variant<QString, Unit> unit);

} // namespace GUI::Util

#endif // BORNAGAIN_GUI_VIEW_TOOL_WIDGETUTILS_H
